open Objets_graphiques;;
open Objets_point_clique;;
open Construction;;
open Fonctions;;
open String;;
open List;;
open GDraw;;
open GtkInit
open GMain;;
open Interface;;

exception ByeBye;;

let repere_demarrage = {xo = (float_of_int largeur_ecran) /. 2.;
			yo = (float_of_int hauteur_ecran) /. 2.;
			xi = (float_of_int largeur_ecran) /. 2. +. 50.;
			yj = (float_of_int hauteur_ecran) /. 2. +. 50.};;

type etat_automate =
  | NONE
  | POINT_CLIQUE
;;

type etat_graphique_automate =
    {mutable couleur_courante : GDraw.color;
     mutable epaisseur_courante : int
    };;

let etat_graphique_demarrage = {couleur_courante = `BLACK;
				epaisseur_courante = 1
			       };;

class type t_automate =
object
  method fenetre : fenetre_principale
  method callback_clavier : GdkEvent.Key.t -> bool
  method callback_souris : GdkEvent.Button.t -> bool
  method callback_expose : GdkEvent.Expose.t -> bool
  method incremente_numero_point : unit
  method incremente_numero_droite : unit
  method incremente_numero_cercle : unit
  method incremente_numero_vecteur : unit
  method reinitialise_numero_point : unit
  method reinitialise_numero_droite : unit
  method reinitialise_numero_cercle : unit
  method reinitialise_numero_vecteur : unit
					   
  method nom_nouveau_point : string
  method nom_nouveau_droite : string
  method nom_nouveau_cercle : string
  method nom_nouveau_vecteur : string

  method repere_automate : t_repere
  method change_repere : t_repere -> unit

  method evenement_clavier : string -> unit
  method evenement_souris : t_coords_int -> unit
    
  method ajouter_objet_liste : t_objet_graphique ref -> unit
  method enlever_objet_liste : t_objet_graphique ref -> unit
  method redessine_figure : unit -> unit
end;;  

class automate =
object (self)
  val mutable liste_construction = ([] : t_construction_list)
  val mutable etat = NONE
  val mutable datas = ([] : t_construction_list)
  val mutable etat_graphique = etat_graphique_demarrage
  val mutable repere = repere_demarrage
  val mutable numero_point = 1
  val mutable numero_droite = 1
  val mutable numero_cercle = 1
  val mutable numero_vecteur = 1

  val mutable fenetre = new fenetre_principale ()
  method fenetre = fenetre

  method callback_clavier ev =
    self#evenement_clavier (GdkEvent.Key.string ev); 
    false
  method callback_souris ev =
    print_string "on est dans callback_souris\n";
    print_string " : ";
    print_int (GdkEvent.Button.button ev);
    print_string "\n x : ";
    print_int (int_of_float (GdkEvent.Button.x ev));
    print_string "\n y : ";
    print_int (int_of_float (GdkEvent.Button.y ev));
    print_newline ();
    match GdkEvent.Button.button ev with
      | 1 ->
	  let x = GdkEvent.Button.x ev
	  and y = GdkEvent.Button.y ev in
	  let c = {xint = int_of_float x; yint = int_of_float y} in
	    self#evenement_souris c;
	    true
      | _ -> false
  method callback_expose t =
    self#redessine_figure ();
    false
  method incremente_numero_point = numero_point <- numero_point + 1
  method incremente_numero_droite = numero_droite <- numero_droite + 1
  method incremente_numero_cercle = numero_cercle <- numero_cercle + 1
  method incremente_numero_vecteur = numero_vecteur <- numero_vecteur + 1
  method reinitialise_numero_point = numero_point <- 1
  method reinitialise_numero_droite = numero_droite <- 1
  method reinitialise_numero_cercle = numero_cercle <- 1
  method reinitialise_numero_vecteur = numero_vecteur <- 1
					
  method nom_nouveau_point =
    String.concat ("_") (["point" ; string_of_int numero_point])
  method nom_nouveau_droite =
    String.concat ("_") (["droite" ; string_of_int numero_droite])
  method nom_nouveau_cercle =
    String.concat ("_") (["cercle" ; string_of_int numero_cercle])
  method nom_nouveau_vecteur =
    String.concat ("_") (["vecteur" ; string_of_int numero_cercle])

  method repere_automate = repere
  method change_repere rep = repere <- rep

  method ajouter_objet_liste obj =
    liste_construction <- ajouter_objet obj liste_construction      
  method enlever_objet_liste obj =
    liste_construction <- enlever_objet obj liste_construction      
  method redessine_figure () =
    let drawing = new GDraw.drawable (self#fenetre#area#misc#window)
    in
      print_string "on est dans redessine";
      print_newline ();
      (* on efface l'cran *)
      drawing#set_background `WHITE;
      drawing#rectangle ~x:0 ~y:0 ~width:largeur_ecran ~height:hauteur_ecran ();
      (* puis on demande  la liste de se redessiner *)
      redessine drawing self#repere_automate liste_construction
      
  method evenement_clavier (key: string) =
    begin
      match key with
	| "\027" ->
	    (* Touche Escape, change l'tat en NONE, *)
	    (* et change NONE en OUINON *)
	    begin
	      match etat with
		| NONE -> ()
		| _ ->
		    etat <- NONE;
		    datas <- [];
		    print_string "Etat : aucun";
		    print_newline ()
	    end
	| "P" | "p" ->
	    (* Touche P, change l'tat en POINT_CLIQUE *)
	    begin
	      match etat with
		| POINT_CLIQUE -> ()
		| _ ->
		    etat <- POINT_CLIQUE;
		    datas <- [];
		    print_string "Etat : point choisi  la souris";
		    print_newline ()
	    end
	| "+" -> 
	    (* Touche +, zoom de sqrt(2) *)
	    begin
	      let sqrt_2 = sqrt 2. in
	      let lef = float_of_int largeur_ecran
	      and hef = float_of_int hauteur_ecran in
	      let rep =
		{xo = (self#repere_automate.xo -. lef /. 2.)
		      *. sqrt_2 +. lef /. 2.;
		 yo = (self#repere_automate.yo -. hef /. 2.) *.
			sqrt_2 +. hef /. 2.;
		 xi = (self#repere_automate.xi -. lef /. 2.) *.
			sqrt_2 +. lef /. 2.;
		 yj = (self#repere_automate.yj -. hef /. 2.) *.
			sqrt_2 +. hef /. 2.;}
	      in
		self#change_repere rep;
		self#redessine_figure ()
	    end
	| "-" -> 
	    (* Touche -, zoom de 1/sqrt(2) *)
	    begin
	      let lef = float_of_int largeur_ecran
	      and hef = float_of_int hauteur_ecran
	      and un_sur_sqrt_2 = 1. /.(sqrt 2.) in
	      let rep =
		{xo = (self#repere_automate.xo -. lef /. 2.) *.
			un_sur_sqrt_2 +. lef /. 2.;
		 yo = (self#repere_automate.yo -. hef /. 2.) *.
			un_sur_sqrt_2	+. hef /. 2.;
		 xi = (self#repere_automate.xi -. lef /. 2.) *.
			un_sur_sqrt_2 +. lef /. 2.;
		 yj = (self#repere_automate.yj -. hef /. 2.) *.
			un_sur_sqrt_2 +. hef /. 2.;}
	      in
		self#change_repere rep;
		self#redessine_figure ()
	    end
	| "8" -> 
	    (* Touche 8, un quart d'cran vers le haut *)
	    begin
	      let hef = float_of_int hauteur_ecran in
	      let rep =
		{xo = (self#repere_automate.xo);
		 yo = (self#repere_automate.yo +. hef /. 4.);
		 xi = (self#repere_automate.xi);
		 yj = (self#repere_automate.yj +. hef /. 4.);}
	      in
		self#change_repere rep;
		self#redessine_figure ()
	    end
	| "2" -> 
	    (* Touche 2, un quart d'cran vers le bas *)
	    begin
	      let hef = float_of_int hauteur_ecran in
	      let rep =
		{xo = (self#repere_automate.xo);
		 yo = (self#repere_automate.yo -. hef /. 4.);
		 xi = (self#repere_automate.xi);
		 yj = (self#repere_automate.yj -. hef /. 4.);}
	      in
		self#change_repere rep;
		self#redessine_figure ()
	    end
	| "" -> 
	    (* Touche Left, un quart d'cran vers la gauche *)
	    begin
	      let lef = float_of_int largeur_ecran in
	      let rep =
		{xo = (self#repere_automate.xo -. lef /. 4.);
		 yo = (self#repere_automate.yo );
		 xi = (self#repere_automate.xi -. lef /. 4.);
		 yj = (self#repere_automate.yj);}
	      in
		self#change_repere rep;
		self#redessine_figure ()
	    end
	| "_Right" -> 
	    (* Touche Right, un quart d'cran vers la droite *)
	    begin
	      let lef = float_of_int largeur_ecran in
	      let rep =
		{xo = (self#repere_automate.xo +. lef /. 4.);
		 yo = (self#repere_automate.yo );
		 xi = (self#repere_automate.xi +. lef /. 4.);
		 yj = (self#repere_automate.yj);}
	      in
		self#change_repere rep;
		self#redessine_figure ()
	      end
	| _ -> ()
    end

  method evenement_souris (ev: t_coords_int) =
    begin
      match etat with
	| POINT_CLIQUE ->
	    let p = ref (new objet_point_clique repere ev)
	    in
	      !p#nomme self#nom_nouveau_point;
	      !p#montre_objet;
	      !p#change_couleur etat_graphique.couleur_courante;
	      !p#change_epaisseur etat_graphique.epaisseur_courante;
	      self#ajouter_objet_liste p;
	      self#incremente_numero_point ;
	      print_string "on appelle redessine";
	      print_newline ();
	      self#redessine_figure ()
	| _ -> ()
    end

  initializer
    self#fenetre#area#misc#realize ();
    self#fenetre#window#event#connect#key_press ~callback:self#callback_clavier;
    self#fenetre#area#event#connect#expose ~callback:self#callback_expose;
    self#fenetre#area#event#connect#button_press ~callback:self#callback_souris;
    self#fenetre#itemquit#connect#activate ~callback:Main.quit;
    self#fenetre#window#connect#destroy ~callback:Main.quit;
    self#fenetre#window#show ();
    Main.main ()      
end;;

