open List;;

exception Bizarre

(* Types de base des agrgats de rels et d'entiers *)
type t_coords = {mutable x : float; mutable y : float};;

type t_coords_int = {mutable xint : int; mutable yint : int};;

type t_equation_droite = {mutable origx : float;
			  mutable origy : float;
			  mutable dirx : float;
			  mutable diry : float};;

type t_equation_cercle = {mutable centrex : float;
			  mutable centrey : float;
			  mutable rayon : float};;

type t_equation =
  | DROITE of t_equation_droite
  | CERCLE of t_equation_cercle;;

type t_repere =
    {mutable xo : float;
     mutable yo : float;
     mutable xi : float;
     mutable yj : float};;

let largeur_ecran = 800;;
let hauteur_ecran = 600;;
let pixels_proche = 49;;

(* Fonctions de transformation des systmes de coordonnes *)
let papier_vers_ecran rep c =
  { xint = int_of_float (c.x *. (rep.xi -. rep.xo) +. rep.xo);
    yint = int_of_float (c.y *. (rep.yj -. rep.yo) +. rep.yo)};;

let ecran_vers_papier rep c =
  { x = (float_of_int c.xint -. rep.xo) /. (rep.xi -. rep.xo);
    y = (float_of_int c.yint -. rep.yo) /. (rep.yj -. rep.yo)};;

let distance_vers_papier rep z = (float_of_int z) /. (rep.xi -. rep.xo);;
let distance_vers_ecran rep z = int_of_float(z *. (rep.xi -. rep.xo));;

(* evaluation du produit vectoriel AM^u *)
(* le signe donne le demi-plan dans lequel se trouve M *)
let coin_sud_ouest rep = ecran_vers_papier rep {xint = 0; yint = 0};;
let coin_nord_est rep =  ecran_vers_papier rep {xint = largeur_ecran; yint = hauteur_ecran};;

let x_min_ecran rep = (coin_sud_ouest rep).x;;
let x_max_ecran rep = (coin_nord_est rep).x;;
let y_min_ecran rep = (coin_sud_ouest rep).y;;
let y_max_ecran rep = (coin_nord_est rep).y;;

let solution_equation a1 b1 c1 a2 b2 c2 =
  let un_sur_delta = 1. /. (a1 *. b2 -. a2 *. b1)
  and numx = c1 *. b2 -. c2 *. b1
  and numy = a1 *. c2 -. a2 *. c1
  in
    {x = numx *. un_sur_delta; y = numy *. un_sur_delta};;
