open Construction;;
open Fonctions;;

exception Erreur of string;;

type t_type_objet =
  | TYPE_AUCUN
  | TYPE_POINT
  | TYPE_DROITE
  | TYPE_VECTEUR
  | TYPE_CERCLE
  | TYPE_REPERE;;

type style_point =
  | CROIX
  | ROND
;;

type style_droite =
  | PLEINE
  | POINTILLEE
;;

type style_objet =
  | BLOP
  | STYLE_POINT of style_point
  | STYLE_DROITE of style_droite

type t_donnees_objet =
    {mutable coords : t_coords list; mutable equation : t_equation list};;

class type virtual t_objet_graphique = object
  method objet_nom : string
  method nomme : string -> unit
  method objet_couleur : GDraw.color
  method change_couleur : GDraw.color -> unit
  method objet_visible : bool
  method montre_objet : unit
  method cache_objet : unit
  method objet_calculable : bool
  method rend_calculable : unit
  method rend_incalculable : unit
  method objet_epaisseur : int
  method change_epaisseur : int -> unit
  method objet_deplace : bool
  method deplace_objet : unit
  method fixe_objet : unit
  method marque_a_supprimer : unit
  method est_a_supprimer : bool
  method change_style : style_objet -> unit
  method parametres : t_donnees_objet
  method change_parametres : t_donnees_objet -> unit
				       
  method virtual objet_type : t_type_objet
  method virtual dessine : [ `window] GDraw.drawable -> t_repere -> unit
  method virtual est_proche : t_repere -> t_coords_int -> bool
  method virtual recalcule : unit
  method virtual depend_de : t_objet_graphique ref -> bool
  method virtual dependance : t_objet_graphique ref list
  method virtual bouge : t_repere -> t_coords_int -> unit
end;;

class virtual objet_graphique =
object
  val mutable nom = ""
  method objet_nom = nom
  method nomme n = nom <- n
		     
  val mutable style = BLOP
  method change_style sty = style <- sty

  val mutable couleur = (`BLACK: GDraw.color)
  method objet_couleur = couleur
  method change_couleur c = couleur <- c
		     
  val mutable visible = true
  method objet_visible = visible
  method montre_objet = visible <- true
  method cache_objet = visible <- false
		     
  val mutable calculable = true
  method objet_calculable = calculable
  method rend_calculable = calculable <- true
  method rend_incalculable = calculable <- false
		     
  val mutable epaisseur = 1
  method objet_epaisseur = epaisseur
  method change_epaisseur e = epaisseur <- e
		     
  val mutable a_bouge = false
  method objet_deplace = a_bouge
  method deplace_objet = a_bouge <- true
  method fixe_objet = a_bouge <- false
		     
  val mutable a_supprimer = false
  method marque_a_supprimer = a_supprimer <- true
  method est_a_supprimer = a_supprimer

  val mutable donnees = {coords = []; equation = []}
  method parametres = donnees
  method change_parametres datas = donnees <- datas

  method virtual objet_type : t_type_objet
  method virtual dessine : [ `window] GDraw.drawable -> t_repere -> unit
  method virtual est_proche : t_repere -> t_coords_int -> bool
  method virtual recalcule : unit
  method virtual depend_de : t_objet_graphique ref -> bool
  method virtual dependance : t_objet_graphique ref list
  method virtual bouge : t_repere -> t_coords_int -> unit
end;;

let premieres_coordonnees_point_objet o =
  match !o#parametres.coords with
    | c :: reste -> c
    | [] -> raise Bizarre;;

let secondes_coordonnees_point_objet o =
  match !o#parametres.coords with
    | c1 :: c2 :: reste -> c2
    | [] -> raise Bizarre;;

let premieres_coordonnees_second_point l =
  match l with
    | a :: b :: q ->
	begin
	  match !b#parametres.coords with
	    | c :: reste -> c
	    | [] -> raise Bizarre
	end
    | _ -> raise Bizarre;;

let premiere_equation_droite_objet o =
  match !o#parametres.equation with
    | DROITE eq :: reste -> eq
    | [] -> raise Bizarre;;

